from .base import BaseModel, ObjectListModel

class MatchList(ObjectListModel):

    def __init__(self):
        super().__init__(list_object=Match)

class Match(BaseModel):

    def __init__(self,
        ID=None,
        EventID=None,
        Round=None,
        Number=None,
        Player1ID=None,
        Score1=None,
        Walkover1=None,
        Player2ID=None,
        Score2=None,
        Walkover2=None,
        WinnerID=None,
        Unfinished=None,
        onBreak=None,
        Status=None,
        WorldSnookerID=None,
        LiveUrl=None,
        DetailsUrl=None,
        PointsDropped=None,
        ShowCommonNote=None,
        Estimated=None,
        Type=None,
        TableNo=None,
        VideoURL=None,
        InitDate=None,
        ModDate=None,
        StartDate=None,
        EndDate=None,
        ScheduledDate=None,
        FrameScores=None,
        Sessions=None,
        Note=None,
        ExtendedNote=None
    ):

        super().__init__()

        self.ID = ID
        self.EventID = EventID
        self.Round = Round
        self.Number = Number
        self.Player1ID = Player1ID
        self.Score1 = Score1
        self.Walkover1 = Walkover1
        self.Player2ID = Player2ID
        self.Score2 = Score2
        self.Walkover2 = Walkover2
        self.WinnerID = WinnerID
        self.Unfinished = Unfinished
        self.onBreak = onBreak
        self.Status = Status
        self.WorldSnookerID = WorldSnookerID
        self.LiveUrl = LiveUrl
        self.DetailsUrl = DetailsUrl
        self.PointsDropped = PointsDropped
        self.ShowCommonNote = ShowCommonNote
        self.Estimated = Estimated
        self.Type = Type
        self.TableNo = TableNo
        self.VideoURL = VideoURL
        self.InitDate = InitDate
        self.ModDate = ModDate
        self.StartDate = StartDate
        self.EndDate = EndDate
        self.ScheduledDate = ScheduledDate
        self.FrameScores = FrameScores
        self.Sessions = Sessions
        self.Note = Note
        self.ExtendedNote = ExtendedNote
        


from .base import BaseModel, ObjectListModel

class PlayerList(ObjectListModel):

    def __init__(self):
        super().__init__(list_object=Player)

class Player(BaseModel):

    def __init__(self,
        ID=None,
        Type=None,
        FirstName=None,
        LastName=None,
        TeamName=None,
        TeamNumber=None,
        TeamSeason=None,
        ShortName=None,
        Nationality=None,
        Sex=None,
        BioPage=None,
        Born=None,
        Twitter=None,
        SurnameFirst=None,
        License=None,
        Club=None,
        URL=None,
        Photo=None,
        PhotoSource=None,
        FirstSeasonAsPro=None,
        LastSeasonAsPro=None,
        Info=None,
        NumRankingTitles=None,
        NumMaximums=None,
        Died=None
    ):

        super().__init__()

        self.ID = ID
        self.Type = Type
        self.FirstName = FirstName
        self.LastName = LastName
        self.TeamName = TeamName
        self.TeamNumber = TeamNumber
        self.TeamSeason = TeamSeason
        self.ShortName = ShortName
        self.Nationality = Nationality
        self.Sex = Sex
        self.BioPage = BioPage
        self.Born = Born
        self.Twitter = Twitter
        self.SurnameFirst = SurnameFirst
        self.License = License
        self.Club = Club
        self.URL = URL
        self.Photo = Photo
        self.PhotoSource = PhotoSource
        self.FirstSeasonAsPro = FirstSeasonAsPro
        self.LastSeasonAsPro = LastSeasonAsPro
        self.Info = Info
        self.NumRankingTitles = NumRankingTitles
        self.NumMaximums = NumMaximums
        self.Died = Died
        


from .base import APIEndpoint

from snookerbot.models.matches import Match, MatchList

class MatchMethods(APIEndpoint):

    def __init__(self, api):
        super().__init__(api, "?t=")

    def get_ongoing(self):
        url = '{endpoint}7'.format(endpoint=self.endpoint)

        status, headers, resp = self.api.get(url)
        if status == 400: return MatchList().parse_error(resp)
        matches = MatchList().parse(resp)
        
        return matches
    
    def get_upcoming(self):
        url = '{endpoint}14'.format(endpoint=self.endpoint)

        status, headers, resp = self.api.get(url)
        if status == 400: return MatchList().parse_error(resp)
        matches = MatchList().parse(resp)
        
        return matches
    
    def get_event(self, event):
        url = '{endpoint}6&e={event}'.format(endpoint=self.endpoint, event=event)

        status, headers, resp = self.api.get(url)
        if status == 400: return MatchList().parse_error(resp)
        matches = MatchList().parse(resp)
        
        return matches


from .base import BaseModel, ObjectListModel

class EventList(ObjectListModel):

    def __init__(self):
        super().__init__(list_object=Event)

class Event(BaseModel):

    def __init__(self,
        ID=None,
        Name=None,
        StartDate=None,
        EndDate=None,
        Sponsor=None,
        Season=None,
        Type=None,
        Num=None,
        Venue=None,
        City=None,
        Country=None,
        Discipline=None,
        Main=None,
        Sex=None,
        AgeGroup=None,
        Url=None,
        Related=None,
        Stage=None,
        ValueType=None,
        ShortName=None,
        WorldSnookerId=None,
        RankingType=None,
        EventPredictionID=None,
        Team=None,
        Format=None,
        Twitter=None,
        HashTag=None,
        ConversionRate=None,
        AllRoundsAdded=None,
        PhotoURLs=None,
        NumCompetitors=None,
        NumUpcoming=None,
        NumActive=None,
        NumResults=None,
        Note=None,
        CommonNote=None,
        DefendingChampion=None,
        PreviousEdition=None,
        Tour=None
    ):

        super().__init__()

        self.ID = ID
        self.Name = Name
        self.StartDate = StartDate
        self.EndDate = EndDate
        self.Sponsor = Sponsor
        self.Season = Season
        self.Type = Type
        self.Num = Num
        self.Venue = Venue
        self.City = City
        self.Country = Country
        self.Discipline = Discipline
        self.Main = Main
        self.Sex = Sex
        self.AgeGroup = AgeGroup
        self.Url = Url
        self.Related = Related
        self.Stage = Stage
        self.ValueType = ValueType
        self.ShortName = ShortName
        self.WorldSnookerId = WorldSnookerId
        self.RankingType = RankingType
        self.EventPredictionID = EventPredictionID
        self.Team = Team
        self.Format = Format
        self.Twitter = Twitter
        self.HashTag = HashTag
        self.ConversionRate = ConversionRate
        self.AllRoundsAdded = AllRoundsAdded
        self.PhotoURLs = PhotoURLs
        self.NumCompetitors = NumCompetitors
        self.NumUpcoming = NumUpcoming
        self.NumActive = NumActive
        self.NumResults = NumResults
        self.Note = Note
        self.CommonNote = CommonNote
        self.DefendingChampion = DefendingChampion
        self.PreviousEdition = PreviousEdition
        self.Tour = Tour


from .base import BaseModel, ObjectListModel

class RoundList(ObjectListModel):

    def __init__(self):
        super().__init__(list_object=Round)

class Round(BaseModel):

    def __init__(self,
        Round=None,
        RoundName=None,
        EventID=None,
        MainEvent=None,
        Distance=None,
        NumLeft=None,
        NumMatches=None,
        Note=None,
        ValueType=None,
        Rank=None,
        Money=None,
        SeedGetsHalf=None,
        ActualMoney=None,
        Currency=None,
        ConversionRate=None,
        Points=None,
        SeedPoints=None
    ):

        super().__init__()

        self.Round = Round
        self.RoundName = RoundName
        self.EventID = EventID
        self.MainEvent = MainEvent
        self.Distance = Distance
        self.NumLeft = NumLeft
        self.NumMatches = NumMatches
        self.Note = Note
        self.ValueType = ValueType
        self.Rank = Rank
        self.Money = Money
        self.SeedGetsHalf = SeedGetsHalf
        self.ActualMoney = ActualMoney
        self.Currency = Currency
        self.ConversionRate = ConversionRate
        self.Points = Points
        self.SeedPoints = SeedPoints
                


from .base import APIEndpoint

from snookerbot.models.players import Player, PlayerList

class PlayerMethods(APIEndpoint):

    def __init__(self, api):
        super().__init__(api, "?p=")

    def get(self, id):
        url = '{endpoint}{id}'.format(endpoint=self.endpoint, id=id)

        status, headers, resp = self.api.get(url)
        if status == 400: return Player().parse_error(resp)
        player = Player().parse(resp[0])
        
        return player


from dateutil import parser

class BaseModel:
    
    def __init__(self):

        self.has_error = False
        self.error = None

    def parse(self, json):
        for key, value in json.items():
            lower_key = ''.join(e.lower() for e in key if e.isalnum())
            lower_attrs = { k.replace('_', '').lower() : k for k in self.__dict__.keys() }

            if lower_key in lower_attrs.keys():
                key = lower_attrs[lower_key]

                try:
                    attr_val = getattr(self, key)

                    if isinstance(attr_val, BaseModel):
                        setattr(self, key, attr_val.parse(value))
                    else:

                        # Low effort checking
                        # If the returned key has "Date" in it, we must parse it to a datetime object
                        if('date' in lower_key):
                            try:
                                value = parser.isoparse(value)
                                setattr(self, key, value)
                            except (parser.ParserError, TypeError, ValueError):
                                pass
                        else:
                            if type(value) == str:
                                if value != '':
                                    setattr(self, key, value)
                            elif type(value) == int:
                                if value != 0:
                                    setattr(self, key, value)
                            else:
                                setattr(self, key, value)

                except AttributeError:
                    pass

        return self
    
    def get_JSON(self):

        dikt = {}
        for k, v in self.__dict__.items():
            if v:
                if isinstance(v, BaseModel):
                    json = v.get_JSON()
                    if json: dikt[k] = json
                else:
                    dikt[k] = v

        return dikt if len(dikt) > 0 else None
    
    def parse_error(self, json):

        from .errors import Error
        
        self.has_error = True
        self.error = Error().parse(json)

        return self

class ObjectListModel(BaseModel):

    def __init__(self, list=None, list_object=None):
        super().__init__()

        self.list = list if list else []
        self.list_object = list_object
    
    def add(self, item):
        self.list.append(item)
        return self.list
    
    def remove(self, item):
        self.list.remove(item)
        return self.list
    
    def parse(self, json):

        if isinstance(json, dict):
            itemObj = self.list_object().parse(json)
            self.add(itemObj)
        elif isinstance(json, list):
            for item in json:
                itemObj = self.list_object().parse(item)
                self.add(itemObj)

        return self
    
    def get_JSON(self):
        list = []

        for item in self.list:
            list.append(item.get_JSON())
        
        return list if len(list) > 0 else None

    def items(self):
        return self.list


import requests
import json

from snookerbot.config import BASE_URL

from snookerbot.endpoints.players import PlayerMethods
from snookerbot.endpoints.matches import MatchMethods
from snookerbot.endpoints.events import EventMethods

class SnookerOrgAPI:

    def __init__(self, requested_by):

        self.requested_by = requested_by

        self.headers = {
            'Accept' : 'application/json',
            'Content-Type' : 'application/json',
            'X-Requested-By' : self.requested_by
        }

        self.base_url = BASE_URL

        self.players = PlayerMethods(self)
        self.matches = MatchMethods(self)
        self.events = EventMethods(self)

    def do_request(self, method, url, data=None, headers=None, files=None):

        if headers:
            merged_headers = self.headers
            merged_headers.update(headers)
            headers = merged_headers
        else: headers = self.headers

        req_url = '{base}/{url}'.format(base=self.base_url, url=url)

        if method == 'GET':
            response = requests.get(req_url, params=data, headers=headers, verify=False)
        elif method == 'POST':
            if files: response = requests.post(req_url, data=json.dumps(data), files=files, headers=headers, verify=False)
            else: response = requests.post(req_url, data=json.dumps(data), headers=headers, verify=False)
        elif method == 'PUT':
            response = requests.put(req_url, data=json.dumps(data), headers=headers, verify=False)
        elif method == 'DELETE':
            response = requests.delete(req_url, params=json.dumps(data), headers=headers, verify=False)

        return response

    def request(self, method, url, data=None, headers=None, files=None):
        response = self.do_request(method, url, data, headers, files)
        resp_content = json.loads(response.content) if response.content else None

        return response.status_code, response.headers, resp_content

    def get(self, url, data=None, headers=None):
        status, headers, response = self.request('GET', url, data, headers)
        return status, headers, response
    
    def post(self, url, data=None, headers=None, files=None):
        status, headers, response = self.request('POST', url, data, headers, files)
        return status, headers, response
    
    def put(self, url, data=None, headers=None):
        status, headers, response = self.request('PUT', url, data, headers)
        return status, headers, response
    
    def delete(self, url, data=None, headers=None):
        status, headers, response = self.request('DELETE', url, data, headers)
        return status, headers, response


from .base import BaseModel

class Error(BaseModel):

    def __init__(self,
        message=None,
        description=None,
        code=None
    ):

        self.message = message
        self.description = description
        self.code = code


from setuptools import setup

# read the contents of README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'python-snookerbot',         
  packages=['snookerbot', 'snookerbot.models', 'snookerbot.endpoints'],
  version = '1.2.0',
  license='GPL-3.0-or-later',
  description = 'Snookerbot',
  long_description=long_description,
  long_description_content_type='text/markdown',
  author = 'Alexander Schillemans',
  author_email = 'alexander.schillemans@hotmail.com',
  url = 'https://github.com/alexander-schillemans/python-snookerbot',
  download_url = 'https://github.com/alexander-schillemans/python-snookerbot/archive/refs/tags/1.2.0.tar.gz',
  keywords = ['snooker', 'snooker.org', 'bot', 'snookerbot', 'api'],
  install_requires=[
          'requests',
          'python-dateutil'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Programming Language :: Python :: 3.10',
  ],
)


import api

snooker_api = api.SnookerOrgAPI('alexander.schillemans@hotmail.com')

# mark_williams = snooker_api.players.get(1)

# print(mark_williams.FirstSeasonAsPro)
# print(mark_williams.LastSeasonAsPro)

# matches = snooker_api.matches.get_event(398)

# for match in matches.items():
#     print(match.ID)

events = snooker_api.events.get_in_season(2022)
for event in events.items():
    print(event.Name)
    date = event.StartDate.strftime('%d/%m/%Y')
    print(date)


from .base import APIEndpoint

from snookerbot.models.events import Event, EventList
from snookerbot.models.rounds import RoundList, Round

class EventMethods(APIEndpoint):

    def __init__(self, api):
        super().__init__(api, "")

    def get(self, id):
        url = '?e={id}'.format(id=id)

        status, headers, resp = self.api.get(url)
        if status == 400: return Event().parse_error(resp)
        event = Event().parse(resp[0])
        
        return event
    
    def get_in_season(self, season):
        url = '?t=5&s={season}'.format(season=season)

        status, headers, resp = self.api.get(url)
        if status == 400: return EventList().parse_error(resp)
        events = EventList().parse(resp)
        
        return events
    
    def get_rounds(self, id):
        url = '?t=12&e={id}'.format(id=id)

        status, headers, resp = self.api.get(url)
        if status == 400: return RoundList().parse_error(resp)
        rounds = RoundList().parse(resp)
        
        return rounds